@@ -10,51 +10,78 @@ use quote::{format_ident, quote};
 pub fn generate<T: Service>(service: &T, proto_path: &str) -> TokenStream {
     let service_ident = quote::format_ident!("{}Client", service.name());
     let client_mod = quote::format_ident!("{}_client", naive_snake_case(service.name()));
-    let mut methods = generate_methods(service, proto_path);
+    let methods = generate_methods(service, proto_path);
 
     let service_doc = generate_doc_comments(service.comment());
 
-    methods.extend(quote! {
-        /// Create a new client using the provided transport.
-        pub fn new_transport(transport: Inner) -> Self {
-            Self {
-                inner: Client::new(transport)
+    let create_methods = quote! {
+        impl<Inner> #service_ident<Inner> {
+            /// Create a new client using the provided transport.
+            pub fn new_transport(transport: Inner) -> Self {
+                Self {
+                    inner: Client::new(transport)
+                }
+            }
+
+            /// Create a new client using the provided generic client.
+            pub fn new_inner(client: Client<Inner>) -> Self {
+                Self {
+                    inner: client,
+                }
             }
         }
+    };
 
-        /// Create a new client using the provided generic client.
-        pub fn new_inner(client: Client<Inner>) -> Self {
-            Self {
-                inner: client,
-            }
-        }
-    });
+    #[allow(unused_mut)]
+    let mut def_transport_impl = TokenStream::new();
 
-    #[allow(unused_variables)]
-    let def_transport_impl = TokenStream::new();
-
-    #[cfg(feature = "default_transport_http")]
-    let def_transport_impl = quote! {
-        use hrpc::{client::transport::http::Hyper, exports::http::Uri};
+    #[cfg(feature = "client_default_transport_hyper_http")]
+    def_transport_impl.extend(quote! {
+        use hrpc::{client::transport::http::{Hyper, HyperError}, exports::http::Uri};
 
         impl #service_ident<Hyper> {
             /// Create a new client using HTTP transport.
             ///
-            /// Panics if the passed URI is invalid.
-            pub fn new<U>(server: U) -> ClientResult<Self, <Hyper as Transport>::Error>
+            /// Panics if the passed URI is an invalid URI.
+            pub fn new<U>(server: U) -> ClientResult<Self, HyperError>
             where
                 U: TryInto<Uri>,
                 U::Error: Debug,
             {
                 let transport =
                     Hyper::new(server.try_into().expect("invalid URL"))
-                        .map_err(ClientError::Transport)?;
+                        .map_err(TransportError::from)
+                        .map_err(ClientError::from)?;
                 Ok(Self {
                     inner: Client::new(transport),
                 })
             }
         }
-    };
+    });
+
+    #[cfg(feature = "client_default_transport_wasm_http")]
+    def_transport_impl.extend(quote! {
+        use hrpc::{client::transport::http::{Wasm, WasmError}, exports::http::Uri};
+
+        impl #service_ident<Hyper> {
+            /// Create a new client using HTTP transport.
+            ///
+            /// Panics if the passed URI is an invalid URI.
+            pub fn new<U>(server: U) -> ClientResult<Self, WasmError>
+            where
+                U: TryInto<Uri>,
+                U::Error: Debug,
+            {
+                let transport =
+                    Wasm::new(server.try_into().expect("invalid URL"))
+                        .map_err(TransportError::from)
+                        .map_err(ClientError::from)?;
+                Ok(Self {
+                    inner: Client::new(transport),
+                })
+            }
+        }
+    });
 
     quote! {
         /// Generated client implementations.
@@ -64,14 +91,19 @@ pub fn generate<T: Service>(service: &T, proto_path: &str) -> TokenStream {
 
             #service_doc
             #[derive(Debug, Clone)]
-            pub struct #service_ident<Inner: Transport> {
+            pub struct #service_ident<Inner> {
                 inner: Client<Inner>,
             }
 
-            impl<Inner: Transport> #service_ident<Inner> {
+            impl<Inner, InnerErr> #service_ident<Inner>
+            where
+                Inner: Service<TransportRequest, Response = TransportResponse, Error = TransportError<InnerErr>> + 'static,
+                InnerErr: 'static,
+            {
                 #methods
             }
 
+            #create_methods
             #def_transport_impl
         }
     }
@@ -97,7 +129,7 @@ fn generate_methods<T: Service>(service: &T, proto_path: &str) -> TokenStream {
             (false, false) => generate_unary,
             (true, true) => generate_streaming,
             (false, true) => generate_server_streaming,
-            (true, false) => panic!("{}: Client streaming server unary method is invalid.", path),
+            (true, false) => generate_client_streaming,
         };
 
         stream.extend(generate_doc_comments(method.comment()));
@@ -112,13 +144,13 @@ fn generate_unary<T: Method>(method: &T, proto_path: &str, path: String) -> Toke
     let (request, response) = method.request_response_name(proto_path);
 
     quote! {
-        pub async fn #ident<Req>(&mut self, req: Req) -> ClientResult<Response<#response>, Inner::Error>
+        pub fn #ident<Req>(&mut self, req: Req) -> impl Future<Output = ClientResult<Response<#response>, InnerErr>> + 'static
         where
             Req: IntoRequest<#request>,
         {
             let mut req = req.into_request();
             *req.endpoint_mut() = Cow::Borrowed(#path);
-            self.inner.execute_request(req).await
+            self.inner.execute_request(req)
         }
     }
 }
@@ -128,13 +160,29 @@ fn generate_streaming<T: Method>(method: &T, proto_path: &str, path: String) ->
     let (request, response) = method.request_response_name(proto_path);
 
     quote! {
-        pub async fn #ident<Req>(&mut self, req: Req) -> ClientResult<Socket<#request, #response>, Inner::Error>
+        pub fn #ident<Req>(&mut self, req: Req) -> impl Future<Output = ClientResult<Socket<#request, #response>, InnerErr>> + 'static
         where
             Req: IntoRequest<()>,
         {
             let mut req = req.into_request();
             *req.endpoint_mut() = Cow::Borrowed(#path);
-            self.inner.connect_socket(req).await
+            self.inner.connect_socket(req)
+        }
+    }
+}
+
+fn generate_client_streaming<T: Method>(method: &T, proto_path: &str, path: String) -> TokenStream {
+    let ident = format_ident!("{}", method.name());
+    let (request, response) = method.request_response_name(proto_path);
+
+    quote! {
+        pub fn #ident<Req>(&mut self, req: Req) -> impl Future<Output = ClientResult<Socket<#request, #response>, InnerErr>> + 'static
+        where
+            Req: IntoRequest<()>,
+        {
+            let mut req = req.into_request();
+            *req.endpoint_mut() = Cow::Borrowed(#path);
+            self.inner.connect_socket(req)
         }
     }
 }
@@ -144,13 +192,13 @@ fn generate_server_streaming<T: Method>(method: &T, proto_path: &str, path: Stri
     let (request, response) = method.request_response_name(proto_path);
 
     quote! {
-        pub async fn #ident<Req>(&mut self, req: Req) -> ClientResult<Socket<#request, #response>, Inner::Error>
+        pub fn #ident<Req>(&mut self, req: Req) -> impl Future<Output = ClientResult<Socket<#request, #response>, InnerErr>> + 'static
         where
             Req: IntoRequest<#request>,
         {
             let mut req = req.into_request();
             *req.endpoint_mut() = Cow::Borrowed(#path);
-            self.inner.connect_socket_req(req).await
+            self.inner.connect_socket_req(req)
         }
     }
 }
